function [ aperture_func ] = m_gen_PSF_aperture_func( params )
% This function calculates the amplitude of electric field and the
% aberration of the wavefront and returns the complex the electric field at
% the tube lens' aperture plane.

% PARENT: m_gen_PSF_numerical_calc.m
% CHILD: m_gen_PSF_aberration.m

int_method='custom';
%int_method='Simpson';
z_correction_method='variance_delta_d_i';
%z_correction_method='empirical';

global aperture_func_bool;
global x_samp_tube_norm_int y_samp_tube_norm_int        % these are needed in "m_gen_PSF_DFT.m"
persistent x_samp_tube_norm y_samp_tube_norm ro_samp_tube_2 circ    % these are needed in "m_gen_PSF_aberration.m"
persistent dipol_comp scalar_aperture_func constant_ab;     % constant parts of the aperture function

psf_type=params.psf.type;
lambda=params.dye.lambda*1E-9;  % emission wavelength in m
k=2*pi/lambda;   % wave number in "1/m"

switch psf_type
    case 'scalar'
        %PSF_model='custom_obj';            % Fourier transform of electric field at the microscope's frontal aperture
        %PSF_model='custom_tube';           % Fourier transform of electric field at tube lens' aperture
        %PSF_model='Airy';                  % calculates Airy diffraction pattern
        %PSF_model='Gibson and Lanni';       % Airy diffraction including Gibson and Lanni aberration term
        %PSF_model='Pankajakshan';          % Gibson and Lanni PSF model including the apodizing function (sqrt(cos(theta)))
        PSF_model='modified Gibson and Lanni';       % Gibson and Lanni PSF model including the apodizing function and transmission coefficients
    case 'vectorial'
        %PSF_model='custom_obj';
        %PSF_model='custom_tube';
        PSF_model='Richards and Wolf';     % Richards and Wolf PSF model
end

% sources:
% Gibson and Lanni:     "Experimental test of an analytical model of aberration in an oil-immersion objective lens used in three-dimensional light microscopy"
%                       "Focusing of light through a stratified medium: a practical approach for computing microscope point spread functions. Part I: Conventional microscopy"
% Pankajakshan:         "Point-spread function retrieval for fluorescence microscopy"
% Richards and Wolf:    "Super-resolution orientation estimation and localization of fluorescent dipoles using 3-D steerable filters"
% T�r�k and Varga:      "Focusing of light through a stratified medium: a practical approach for computing microscope point spread functions. Part I: Conventional microscopy"
%                       "Is it differs from the "Richards and Wolf" model?"
% Haeberl�:             "Focusing of light through a stratified medium: a practical approach for computing microscope point spread functions. Part I: Conventional microscopy"
%                       "Fluorescence microscopy point spread function model accounting for aberrations due to refractive index variability within a specimen"
%                       "Is it differs from the "Richards and Wolf" model?"

switch PSF_model
    case 'custom_tube'
        Abbe_bool=true;     % boolean for whether the intesity sholund be distorted between the objective's and tube lens' aperture according to ray distribution following Abbe sine condition under desing conditions
    otherwise
        Abbe_bool=false;     % boolean for whether the intesity sholund be distorted between the objective's and tube lens' aperture according to ray distribution following Abbe sine condition under desing conditions
end

if strcmp(psf_type,'vectorial')
    anisotropy_bool=params.psf.anisotropy_bool;
    orientation=params.mol.orientation;
end

aperture_rad_tube=params.numcalc.aperture_rad_tube;     % modified radius of tube lens' aperture in m
aperture_rad_obj=params.numcalc.aperture_rad_obj;       % radius of objective's frontal aperture in m
focal_plane_dist=params.numcalc.focal_plane_dist;       % distance between the sample and the objective's aperture under desing conditions


% function for varying theta angle in the immersion medium in order that
% the ray hit the sampled point on the aperture
    function F=ray_angles(theta, h_ap)
        F=(h_ap-(d_i+delta_d_i)*tan(theta)-...
            d_g*n_i*sin(theta)/n_g./sqrt(1-(n_i*sin(theta)/n_g).^2)-...
            d_s*n_i*sin(theta)/n_s./sqrt(1-(n_i*sin(theta)/n_s).^2))*1E9;
    end

    function [theta, error_ray_angles_iteration, exitflag_ray_angles_iteration]=ray_angles_iteration(theta_edge_i_0, theta_i_max_0, theta_i_min_0, h_ap)
        theta=theta_edge_i_0;
        theta_max=theta_i_max_0;        % maximal angle value, the angle belonging to the edge must be under it
        theta_min=theta_i_min_0;        % minimal angle value, the angle belonging to the edge must be over it
        exitflag_ray_angles_iteration=0;
        delta_x_ap_0=-ray_angles(theta, h_ap);  % initial position change where the ray intersects the aperture
        theta_prev=theta;   % theta of the previous iteration
        for idxTheta=1:100
            theta_prev2=theta_prev; % theta of the 2 previous iteration
            theta_prev=theta;
            delta_x_ap=-ray_angles(theta, h_ap);    % position change where the ray intersects the aperture
            if delta_x_ap<0                         % if the ray intersects the aperture below the edge
                theta_min=theta;
                theta=(5*theta_max+theta_min)/6;    % weighted mean, for faster convergence
            elseif delta_x_ap>0                     % if the ray intersects the aperture above the edge
                theta_max=theta;
                theta=(5*theta_min+theta_max)/6;    % weighted mean, for faster convergence
            elseif delta_x_ap==0                    % if the ray intersects the aperture exactly at the edge
                exitflag_ray_angles_iteration=1;
                error_ray_angles_iteration=(delta_x_ap)/h_ap;
                break;
            end
            
            % if the "theta" change is in the order of the smallest number,
            % without it, it might not converge to it, only oscillate
            % around it
            if abs(theta-theta_prev)==eps
                if abs(delta_x_ap)<=h_ap*1e-4
                    exitflag_ray_angles_iteration=1;
                    break;
                end
            elseif theta==theta_prev2
                if theta>theta_prev
                    theta=theta_prev+eps;
                elseif theta<theta_prev
                    theta=theta_prev-eps;
                end
            end
            
        end
        
        % in this case, consider the iteration successful
        if abs(delta_x_ap)<=h_ap*1e-4 
            exitflag_ray_angles_iteration=1;
        end
        error_ray_angles_iteration=(delta_x_ap)/h_ap;
    end

% error function of the surface (defocus on unit circle) fitting
    function G=defocus_fitting(fit_var)
        G=sum(sum(abs((Lanni_Gibson_ab_orig-(fit_var(1)+fit_var(2)*ro_samp_tube_2)).*circ).^error_power.*error_weights.*int_mat))*tolerance_const;
    end

% function for calculating the radial dependence of PSF during PSF
% size minimalization of PSF distorted by Gibson and Lanni aberration
    function H=PSF_func(delta_d_i_iter, defocus_iter)
        H=abs((besselj(0,2*pi*2.5*0.5*ro_det_var*ro_tube_var')*(ro_tube_var.*apodizing_func_PSF_variance...           % Hankel transform of the approximated aperture function
            .*exp(1i*k*(d_s*sqrt(n_s^2-NA_LG^2*ro_tube_var.^2)+delta_d_i_iter*sqrt(n_i^2-NA_LG^2*ro_tube_var.^2)-defocus_iter*ro_tube_var.^2))))').^2;    % Gibson and Lanni aberration
    end

% function for calculating the variance of PSF during PSF
% size minimalization of PSF disrtorted by Gibson and Lanni aberration
    function I=PSF_variance(delta_d_i_iter)
        defocus_iter=0;                                 % do not add defocus arbitrarily
        I=PSF_func(delta_d_i_iter, defocus_iter)...                           % PSF
            /(PSF_func(delta_d_i_iter, defocus_iter)*2*pi*ro_det_var)...     % normalization
            *(2*pi*ro_det_var.^3)*1E3;                             % multipliing with radius^2 for variance and integrating in polar coordiantes
    end

% function for calculating the variance of PSF during PSF
% size minimalization of PSF disrtorted by Gibson and Lanni aberration
    function J=PSF_defocus_variance(defocus_iter)
        delta_d_i_iter=delta_d_i;
        J=PSF_func(delta_d_i_iter, defocus_iter)...                         % PSF
            /(PSF_func(delta_d_i_iter, defocus_iter)*2*pi*ro_det_var)...     % normalization
            *(2*pi*ro_det_var.^3)*1E3;                         % multipliing with radius^2 for variance and integrating in polar coordiantes
        
    end

    function apodizing_func_approx=apodizing_func_approx_cacl(theta_edge_i, h_ap, PSF_model)
        % Approximating the apodization and reflections to get an
        % approximate normalized electic field values on the
        % objective's aperture.
        % Lets consider the source is in a homogenous medium in the
        % aspect of the propagation direction, so no iteration has
        % to used to calculate the exact ray angles
        
        % lets calculate source's distance to the objective's
        % aperture from the already calculated marginal ray angle
        d_approx=h_ap/tan(theta_edge_i);
        
        % calculate the ray angles in every mediums to be able
        % to consider the transmission coefficients and the
        % apodization effect
        theta_i_approx=atan(h_ap*ro_tube_var/d_approx);
        theta_g_approx=asin(n_i/n_g*sin(theta_i_approx));
        theta_s_approx=asin(n_i/n_s*sin(theta_i_approx));
        
        % transmission coefficient of different surfaces for individual rays
        t_s_sg_approx=2*n_s*cos(theta_s_approx)./(n_s*cos(theta_s_approx)+n_g*cos(theta_g_approx));
        t_p_sg_approx=2*n_s*cos(theta_s_approx)./(n_s*cos(theta_g_approx)+n_g*cos(theta_s_approx));
        t_s_gi_approx=2*n_g*cos(theta_g_approx)./(n_g*cos(theta_g_approx)+n_i*cos(theta_i_approx));
        t_p_gi_approx=2*n_g*cos(theta_g_approx)./(n_g*cos(theta_i_approx)+n_i*cos(theta_g_approx));
        
        % in case of total internal reflection
        t_s_sg_approx(imag(t_s_sg_approx)~=0)=0;
        t_p_sg_approx(imag(t_p_sg_approx)~=0)=0;
        t_s_gi_approx(imag(t_s_gi_approx)~=0)=0;
        t_p_gi_approx(imag(t_p_gi_approx)~=0)=0;
        
        % total transmission coefficients for the individual rays
        t_s_approx=t_s_sg_approx.*t_s_gi_approx;
        t_p_approx=t_p_sg_approx.*t_p_gi_approx;
        
        t_approx=(t_s_approx+t_p_approx)/2;     % the transmission coefficient
        apodization_approx=sqrt(cos(theta_i_approx));   % the apodization effect
        E_aperture_change_approx=d_approx./sqrt(((h_ap*ro_tube_var).^2+d_approx^2));     % electic field deviation between the middle and marginal part of the aperture because of the different distance from the source
        
        % the overall apodization of the electric field
            switch PSF_model
                case {'custom_obj','custom_tube'}
                    apodizing_func_approx=t_approx.*apodization_approx.*E_aperture_change_approx;
                case {'Airy', 'Gibson and Lanni'}
                    apodizing_func_approx=ones(size(ro_tube_var));
                case {'Pankajakshan'}
                    apodizing_func_approx=apodization_approx;
                case {'modified Gibson and Lanni', 'Richards and Wolf'}
                    apodizing_func_approx=t_approx.*apodization_approx;
            end
    end


%ray_angle_iteration_method='matlab';
ray_angle_iteration_method='custom';

%apodization_approx_method='empirical';
apodization_approx_method='physical approx';

% Calculating constant functions which do not change from molecule to molecule
if ~aperture_func_bool      % if it has not been calculated for the actual dye or the parameters has been changed (generate button has benn pressen once again)
    
    % reading constants
    em_N_ofphotonpersec_mean=params.dye.em_N_ofphotonpersec;        % avarage number of emitted photons per second (avarage of emissions with all possible dipole orientations)
    NA=params.aq.NA;
    sampling_number_x=params.numcalc.sampling_number_x;
    sampling_number_y=params.numcalc.sampling_number_y;
    sampling_index_x=params.numcalc.sampling_index_x;
    sampling_index_y=params.numcalc.sampling_index_y;
    aperture_sampling_fill_bool=params.numcalc.aperture_sampling_fill_bool;
    
    n_i=params.aq.n_immersion;      % refractive index of immersion medium
    n_g=n_i;                        % refractive index of cover slip
    n_s=params.aq.n_sample;        % refractive index of sample
    
    d_i=focal_plane_dist-params.numcalc.z_focal_x;	% thickness of the immersion medium (and the cover slip) under design conditions in m
    d_g=0*1E-6;                          % thickness of the cover slip, it's taken into account in "d_i" in m
    % The aberration because the shift of joint focal point in case of 3D
    % astigmatic measurement is not taken into account because "d_g" is
    % zero and otherwise it would be insignificant in normal case.
    d_s=params.psf.sample_depth*1E-6;          % thickness of the sample layer in m
    
    % determining the maximal ray angles belonging to the total intrenal
    % refraction in the immersion medium
    switch max([n_s, n_g, n_i])
        case n_i    % if the immersion medium has the highest refractive index
            theta_i_max=asin(min([n_s, n_g, n_i])/n_i);
        case n_g    % if the cover slip has the highest refractive index
            if n_i>n_s
                theta_i_max=asin(min([n_s, n_g, n_i])/n_i);
            elseif n_i<=n_s
                theta_i_max=pi/2;
            end
        case n_s    % if the sample medium has the highest refractive index
            if n_i>n_g
                theta_i_max=asin(min([n_s, n_g, n_i])/n_i);
            elseif n_i<=n_g
                theta_i_max=pi/2;
            end
    end
    
    switch PSF_model
        case {'Airy'}
            delta_d_i=-d_s*n_i/n_s;
        otherwise
            switch z_correction_method
                case 'variance_delta_d_i'
                    % Minimizing the PSF variance with immersion thickness change.
                    sampling_number_var=256;
                    
                    ro_tube_var=(0.5:sampling_number_var-0.5)'/sampling_number_var;        % normalized radii values of the tube lens' aperture
                    ro_det_var=(0.5:sampling_number_var-0.5)'/sampling_number_var;         % normalized radii values of the imaged area
                    
                    delta_d_i_0=-d_s*n_i/n_s;               % initial thickness change of the immersion medium
                    delta_d_i=delta_d_i_0;                  % thickness change of the immersion medium
                    
                    theta_max_ratio=0.95;   % starting maximum allowed ray angle and angle of total internal reflection ratio
                    % initial guess for the ray angles belonging to the intersection border points
                    theta_edge_i_0=min(atan(abs(aperture_rad_obj)/(d_i+delta_d_i+d_s)), theta_i_max*theta_max_ratio);     % initial guess for the ray angles, required for the iteration method
                    
                    for var_iter_idx=1:10        % let this the maximal number of iterations for calculating the "z" pos where the PSF variance is minimal
                        var_iter_success_bool=true;    % do not break out of the "while" loop
                        % running the itarition with different "theta_bord_i_0_max" values
                        
                        switch ray_angle_iteration_method
                            case 'matlab'
                                options = optimset('Display','off', 'FinDiffType', 'central');        % do not display iteration output
                                N_fsolve_loop=10;       % maximal number of loops for the iteration of "theta_i_ap_edge", inner loops, should be enough
                                fsolve_loop=1;       % variable for the "while" loop
                                while fsolve_loop<=N_fsolve_loop
                                    iter_success_bool=true;    % boolean for storing whether all the iterations were successful
                                    theta_edge_i_0_max=theta_i_max*theta_max_ratio;            % let this be the maximal initial ray angle, should be smaller than the angle of total internal refraction
                                    % initial ray angle for the iteration
                                    if var_iter_idx==1
                                        theta_edge_i_0_iter=theta_edge_i_0;
                                    else
                                        theta_edge_i_0_iter=theta_edge_i;
                                    end
                                    theta_edge_i_0_iter(theta_edge_i_0_iter>theta_edge_i_0_max)=theta_edge_i_0_max;   % if any of the initial guess exceeds the maximal value...
                                    
                                    [theta_edge_i, ~, exitflag]=fsolve(@(theta) ray_angles(theta, abs(aperture_rad_obj)), theta_edge_i_0_iter, options);   % iteration for getting the ray angles
                                    % examining whether the iteration was successfull for a given ray
                                    if exitflag>0       % if the iteration was successful
                                    else                % if the iteration was not successful
                                        iter_success_bool=false;    % do not break out of the "while" loop
                                    end
                                    
                                    % if the iterations were successful for all angles...
                                    if iter_success_bool
                                        break;           % break out of the loop
                                    end
                                    % decrease the "theta_bord_i_0_max" and "theta_i_max" difference in
                                    % the next loop
                                    fsolve_loop=fsolve_loop+1;
                                    theta_max_ratio=1-(1-theta_max_ratio)/2;
                                end
                            case 'custom'
                                [theta_edge_i, ~, exitflag]=ray_angles_iteration(theta_edge_i_0, theta_i_max, 0, abs(aperture_rad_obj));
                                % examining whether the iteration was successfull for a given ray
                                if exitflag>0       % if the iteration was successful
                                    iter_success_bool=true;
                                else                % if the iteration was not successful
                                    iter_success_bool=false;
                                end
                        end
                        if ~iter_success_bool
                            theta_edge_i=theta_i_max;
                            %error('Could not calculate ray angle for aperture edge. Please decrease the numerical aperture.')
                        end
                        
                        
                        
                        % approximated apodizing function, gives the "shape" of the aperture function
                        % did not want to calculate it here, did not want to run the expensive
                        % iteration for all ray angles here too and experienced as the the shape of
                        % the aperture function does not affects much the outcome variable value of
                        % minimalization of the PSF
                        switch apodization_approx_method
                            case 'empirical'
                                apodizing_func_PSF_variance=cos(theta_edge_i*ro_tube_var).^1.5;           % this fits relatively well to the product of the apodizing function and the transmission coefficients (modified apodizing function)
                                %apodizing_func=cos(atan(abs(aperture_rad_obj*ro_tube_iter)/(delta_d_i))).^0.5;
                                %apodizing_func_PSF_variance=1;
                            case 'physical approx'
                                apodizing_func_PSF_variance=apodizing_func_approx_cacl(theta_edge_i, abs(aperture_rad_obj), PSF_model);
                        end
                        
                        
                        NA_LG=n_i*sin(theta_edge_i);    % numerical aperture in Gibson and Lanni aberration
                        
                        
                        delta_d_i_0=delta_d_i;      % starting immersion thickness change for the PSF variance calculation
                        
                        [delta_d_i,~,exitflag_var_iter]=fminsearch(@(delta_d_i_iter) PSF_variance(delta_d_i_iter), delta_d_i_0);       % minimizing the PSF variance
                        
                        if exitflag_var_iter>0       % if the iteration was successful
                            if abs((delta_d_i-delta_d_i_0)/delta_d_i_0)<1E-4
                                var_iter_success_bool=true;
                                break;
                            end
                        else                % if the iteration was not successful
                            var_iter_success_bool=false;    % break out of the "for" loop
                            break;
                        end
                        
                    end
                    
                    if ~var_iter_success_bool       % if the iteration was successful
                        error('Could not minimalize the PSF variance. Please decrease the numerical aperture or the simple medium thickness.')                % if the iteration was not successful
                    end
                case 'variance_defocus'
                    % Minimizing the PSF variance with defocus
                    
                    delta_d_i=-d_s*n_i/n_s;
                case 'empirical'
                    % Minimizing the PSf width with defocus determined with
                    % empirical constants.
                    
                    delta_d_i=-d_s*n_i/n_s;
            end
    end
    
    % calculating the border point (vertices of squares around the sampled points) coordinates on the aperture
    border_number_x=sampling_number_x+1;        % number of border points along "x" dimension
    border_number_y=sampling_number_y+1;        % number of border points along "y" dimension
    border_index_x=1:border_number_x;           % index of border points along "x" dimension
    border_index_y=1:border_number_y;           % index of border points along "y" dimension
    
    if aperture_sampling_fill_bool
        x_samp_tube_norm_int=(-1+2*(sampling_index_x'-1)/(sampling_number_x-1));  % vector of normalized x coordinates of sampled points on tube lens' aperture
        y_samp_tube_norm_int=(-1+2*(sampling_index_y-1)/(sampling_number_y-1));  % vector of normalized y coordinates of sampled points on tube lens' aperture
        
        x_bord_tube_norm_int=(-1+2*(border_index_x'-1)/(border_number_x-1))*(border_number_x-1)/(sampling_number_x-1);  % vector of normalized x coordinates of sampled points on tube lens' aperture
        y_bord_tube_norm_int=(-1+2*(border_index_y-1)/(border_number_y-1))*(border_number_y-1)/(sampling_number_y-1);  % vector of normalized y coordinates of sampled points on tube lens' aperture
        
    elseif ~aperture_sampling_fill_bool
        x_samp_tube_norm_int=(-1+2*(sampling_index_x'-1)/(sampling_number_x-1))*(sampling_number_x-1)/sampling_number_x;  % vector of normalized x coordinates of sampled points on tube lens' aperture
        y_samp_tube_norm_int=(-1+2*(sampling_index_y-1)/(sampling_number_y-1))*(sampling_number_y-1)/sampling_number_y;  % vector of normalized y coordinates of sampled points on tube lens' aperture
        
        x_bord_tube_norm_int=(-1+2*(border_index_x'-1)/(border_number_x-1));  % vector of normalized x coordinates of sampled points on tube lens' aperture
        y_bord_tube_norm_int=(-1+2*(border_index_y-1)/(border_number_y-1));  % vector of normalized y coordinates of sampled points on tube lens' aperture
    end
    
    x_samp_tube_norm=repmat(x_samp_tube_norm_int,1,sampling_number_y);     % array containing the sampled points' normalized x coordinates on tube lens' aperture
    y_samp_tube_norm=repmat(y_samp_tube_norm_int,sampling_number_x,1);     % array containing the sampled points' normalized y coordinates on tube lens' aperture
    
    x_bord_tube_norm=repmat(x_bord_tube_norm_int,1,border_number_y);     % array containing the sampled points' normalized x coordinates on tube lens' aperture
    y_bord_tube_norm=repmat(y_bord_tube_norm_int,border_number_x,1);     % array containing the sampled points' normalized y coordinates on tube lens' aperture
    
    % polar angle between the ray and optical axis in the aperture's plane
    % needed for calculating the "s" and "p" components' unit vectors  and
    % the surface elemets
    fi_samp=atan2(y_samp_tube_norm,x_samp_tube_norm);
    
    fi_bord=atan2(y_bord_tube_norm,x_bord_tube_norm);
    
    ro_samp_tube_2=x_samp_tube_norm.^2+y_samp_tube_norm.^2;     % square of normalized radii of the sampled points on tube lens' aperture
    ro_samp_tube=sqrt(ro_samp_tube_2);      % normalized radii of the sampled points on tube lens' aperture
    
    ro_bord_tube_2=x_bord_tube_norm.^2+y_bord_tube_norm.^2;     % square of normalized radii of the sampled points on tube lens' aperture
    ro_bord_tube=sqrt(ro_bord_tube_2);      % normalized radii of the sampled points on tube lens' aperture
    
    circ=zeros(sampling_number_x,sampling_number_y);        % circular function of the aperture
    circ(ro_samp_tube<=1)=1;
    
    if Abbe_bool    % do not distort the intensity distribution between the objective's ang the tube lens' aperture according to Abbe sine condition
        % calculating the sampled point coordinates on the objective's frontal aperture
        r_samp_i=real(focal_plane_dist*tan(asin(NA/n_i*ro_samp_tube)));
        r_samp_i(circ==0)=0;
        
        x_bord_i_int=real(focal_plane_dist*tan(asin(NA/n_i*x_bord_tube_norm_int)));     % "x" coordinates of border points on the "x" axis
        y_bord_i_int=real(focal_plane_dist*tan(asin(NA/n_i*y_bord_tube_norm_int)));     % "y" coordinates of border points on the "x" axis
        
        %figure()
        %plot(ro_samp_tube(64,:), real(focal_plane_dist*tan(asin(NA/n_i*ro_samp_tube(64,:))))./ro_samp_tube(64,:) )
        
        %ro_samp_i=r_samp_i/aperture_rad_obj;
        %ro_samp_i_2=ro_samp_i.^2;
        %x_samp_i_norm=ro_samp_i.*cos(fi_samp);
        %y_samp_i_norm=ro_samp_i.*sin(fi_samp);
        
        % calculating the border point coordinates on the aperture
        r_bord_i=real(focal_plane_dist*tan(asin(NA/n_i*ro_bord_tube)));        % radius of the sampled points
        r_bord_i(circ==0)=0;
    else    % do not distort the intensity distribution between the objective's ang the tube lens' aperture
        % calculating the sampled point coordinates on the objective's frontal aperture
        r_samp_i=ro_samp_tube*aperture_rad_obj;        % radius of the sampled points
        r_samp_i(circ==0)=0;
        
        x_bord_i_int=x_bord_tube_norm_int*aperture_rad_obj;     % "x" coordinates of border points on the "x" axis
        y_bord_i_int=y_bord_tube_norm_int*aperture_rad_obj;     % "y" coordinates of border points on the "x" axis
        
        %ro_samp_i=r_samp_i/aperture_rad_obj;
        %ro_samp_i_2=ro_samp_i.^2;
        %x_samp_i_norm=ro_samp_i.*cos(fi_samp);
        %y_samp_i_norm=ro_samp_i.*sin(fi_samp);
        
        % calculating the border point coordinates on the aperture
        r_bord_i=ro_bord_tube*aperture_rad_obj;        % array containing the radii of border points
    end
    
    % indices for iteration
    iter_number=2*max(border_number_x, border_number_y);    % should be enough dense
    if rem(iter_number,2)==0
        iter_number=iter_number+1;      % let it be odd
    end
    iter_index=1:iter_number;
    
    % calculating border coordinates on the objective's frontal aperture for iteration
    min_bord_norm=min(min(x_bord_i_int), min(y_bord_i_int));
    max_bord_norm=max(max(x_bord_i_int), max(y_bord_i_int));
    iter_coords=(min_bord_norm+(max_bord_norm-min_bord_norm)*(iter_index'-1)/(iter_number-1));
    
    
    theta_max_ratio=0.95;   % starting maximum allowed ray angle and angle of total internal reflection ratio
    % initial guess for the ray angles belonging to the intersection border points
    theta_iter_i_0=min(atan(abs(iter_coords)/(d_i+delta_d_i+d_s)), theta_i_max*theta_max_ratio);     % initial guess for the ray angles, required for the iteration method
    
    % initializing the ray angles belonging to the intersection border points with an iteration method
    theta_bord_i_iter=zeros(iter_number,1);     % ray angles in the immersion medium calculeted with the iteration
    
    iter_idx=true(1,iter_number); % logical ray indices for which the iteration should be performed
    iter_bool=false(1,iter_number);             % boolean storing whether the iteration was successful or not
    
    switch ray_angle_iteration_method
        case 'matlab'
            % running the itarition with different "theta_bord_i_0_max" values
            options = optimset('Display','off', 'FinDiffType', 'central');        % do not display iteration output
            N_fsolve_loop=10;       % maximal number of loops for the iteration of "theta_i_ap_edge", inner loops, should be enough
            fsolve_loop=1;       % variable for the "while" loop
            while fsolve_loop<=N_fsolve_loop
                iter_success_bool=true;    % boolean for storing whether all the iterations were successful
                theta_bord_i_0_max=theta_i_max*theta_max_ratio;            % let this be the maximal initial ray angle, should be smaller than the angle of total internal refraction
                theta_bord_int_i_0_iter=theta_iter_i_0;
                theta_bord_int_i_0_iter(theta_bord_int_i_0_iter>theta_bord_i_0_max)=theta_bord_i_0_max;   % if any of the initial guess exceeds the maximal value...
                
                for act_idx=find(iter_idx)
                    [theta_bord_i_iter(act_idx), ~, exitflag]=fsolve(@(theta) ray_angles(theta, abs(iter_coords(act_idx))), theta_bord_int_i_0_iter(act_idx), options);   % iteration for getting the ray angles
                    % examining whether the iteration was successfull for a given ray
                    if exitflag>0       % if the iteration was successful
                        iter_bool(act_idx)=true;
                        iter_idx(act_idx)=false;   % modify these logical indices
                    else                % if the iteration was not successful
                        iter_bool(act_idx)=false;
                        iter_success_bool=false;    % do not break out of the "while" loop
                    end
                end
                % if the iterations were successful for all angles...
                if iter_success_bool
                    break;           % break out of the loop
                end
                % decrease the "theta_bord_i_0_max" and "theta_i_max" difference in
                % the next loop
                fsolve_loop=fsolve_loop+1;
                theta_max_ratio=1-(1-theta_max_ratio)/2;
            end
        case 'custom'
            for act_idx=find(iter_idx)
                [theta_bord_i_iter(act_idx), ~, exitflag]=ray_angles_iteration(theta_iter_i_0(act_idx), theta_i_max, 0, abs(iter_coords(act_idx)));
                % examining whether the iteration was successfull for a given ray
                if exitflag>0       % if the iteration was successful
                    iter_bool(act_idx)=true;
                    iter_idx(act_idx)=false;   % modify these logical indices
                else                % if the iteration was not successful
                    iter_bool(act_idx)=false;
                end
            end
    end
    % if the iteration was not successfull for the marginal rays let them
    % be the maximal "theta" value
    if ~iter_bool(1) || ~iter_bool(end)     % if the iteration was not successful for the marginal rays...
        theta_bord_i_iter(1)=theta_i_max;	% let them get the value of the maximal angle allowed by the total internal reflection
        theta_bord_i_iter(end)=theta_i_max;	% let them get the value of the maximal angle allowed by the total internal reflection
        iter_bool_mod=iter_bool;            % modified boolean array
        iter_bool_mod(1)=1;
        iter_bool_mod(end)=1;
    else
        iter_bool_mod=iter_bool;
    end
    
    % preparing the vectors containing only the values for which the iteration was successfull
    theta_bord_int_real=theta_bord_i_iter;      % vector containing ray angles for which the iteration was successfull
    theta_bord_int_real(iter_bool_mod==0)=[];   % deleting the elements for which the iteration was unsuccessfull
    iter_coords_real=iter_coords;               % vector containing "x" coordinates for which the iteration was successfull
    iter_coords_real(iter_bool_mod==0)=[];      % deleting the elements for which the iteration was unsuccessfull
    
    % interpolate the ray angles in order to get values for ray angles for
    % which the iteration was not successfull too
    theta_iter_i=real(interp1(iter_coords_real, theta_bord_int_real, iter_coords, 'pchip')); % interpolate the ray angle values for which the iteration was not successful
    theta_iter_i(theta_iter_i>theta_i_max)=theta_i_max;       % just for safety
    
    % should already be zero, just for safety
    if rem(iter_number,2)==1
        theta_iter_i((iter_number+1)/2)=0;
    end
    
    
%     % calculating the border ray angles in the cover slip and the sample medium
%     if n_i/n_s*sin(theta_i_max)>=1   % if there might be total internal refraction on the immersion-sample interface
%         theta_iter_g=asin(n_i/n_g*sin(theta_iter_i));     % ray angles in the cover slip
%         theta_iter_s(iter_bool)=asin(n_i/n_s*sin(theta_iter_i(iter_bool)));	% ray angles in the sample if the iteration was successful
%         % if the ray angle was approximated do not use Snell's law (the iterationusually is not successfull for angles near the total internal reflection)
% %         theta_iter_s(~iter_bool)=atan((abs(iter_coords(~iter_bool))-...	% geometrical approximation of the ray angles in the sample if the iteration was not successfull
% %             (d_i+delta_d_i)*tan(theta_iter_i(~iter_bool))-...
% %             d_g*tan(theta_iter_g(~iter_bool)))/d_s);
%     elseif n_i/n_g*sin(theta_i_max)>=1   % if there might be total internal reflection on the sample-cover slip interface
%         theta_iter_s=asin(n_i/n_s*sin(theta_iter_i));     % ray angles in the sample
%         theta_iter_g(iter_bool)=asin(n_i/n_g*sin(theta_iter_i(iter_bool)));	% ray angles in the cover slip if the iteration was successful
%         % if the ray angle was approximated do not use Snell's law (the iteration usually is not successfull for angles near the total internal reflection)
% %         theta_iter_g(~iter_bool)=atan((abs(iter_coords(~iter_bool))-... % geometrical approximation of the ray angles in the cover slip if the iteration was not successfull
% %             (d_i+delta_d_i)*tan(theta_iter_i(~iter_bool))-...
% %             d_s*tan(theta_iter_s(~iter_bool)))/d_g);
%     else
%         error('Something gone wrong in calculating the maximal ray angle.')
%     end
    
    % calculating the border ray angles in the cover slip and the sample medium
    theta_iter_g=zeros(size(theta_iter_i));
    theta_iter_s=zeros(size(theta_iter_i));
    theta_iter_g(iter_bool)=asin(n_i/n_g*sin(theta_iter_i(iter_bool)));	% ray angles in the cover slip if the iteration was successful
    theta_iter_s(iter_bool)=asin(n_i/n_s*sin(theta_iter_i(iter_bool)));	% ray angles in the sample if the iteration was successful
        
    % if the iteration was not successful for the marginal rays let
    % calcute them with a different approach
    if ~iter_bool(1) || ~iter_bool(end)     % if the iteration was not successful for the marginal rays...
        theta_iter_s(1)=atan((abs(iter_coords(1))-tan(theta_i_max)*(d_i+delta_d_i))/d_s);
        theta_iter_s(end)=atan((abs(iter_coords(end))-tan(theta_i_max)*(d_i+delta_d_i))/d_s);
        iter_bool_mod=iter_bool;            % modified boolean array
        iter_bool_mod(1)=1;
        iter_bool_mod(end)=1;
    else
        iter_bool_mod=iter_bool;
    end
    
    % interpolating the unsuccessful marginal rays in the sample layer
    theta_iter_s(~iter_bool_mod)=interp1(iter_coords(iter_bool_mod), theta_iter_s(iter_bool_mod), iter_coords(~iter_bool_mod), 'cubic');
    
    theta_iter_g(~iter_bool_mod)=asin(n_s/n_g*sin(theta_iter_s(~iter_bool_mod)));
    
    % if the calculation of the ray angles was not successfull after all,
    % the numerical aperture should be decreased
    if ~isreal(theta_iter_i) || ~isreal(theta_iter_g) || ~isreal(theta_iter_s)
        error('Could not calculate the angle of refraction. Please decrease the numerical aperture of the objective.')
    end
    
    % calculating border point coordinates on the sample-immersion
    % interface
    
    % interpolating the ray angles to get them for the whole aperture, not
    % just for the intersection
    % "pchip" (some kind of cubic) interpolation is used because he "spline interpolation fucks up some points on the middle of the graphs"
    theta_bord_s(:,border_index_y)=interp1(iter_coords, theta_iter_s, r_bord_i(:,border_index_y), 'pchip');
    
    % just for safety reasons
    theta_bord_s(theta_bord_s>max(theta_iter_s))=max(theta_iter_s);
    
    ro_bord_si=d_s*tan(theta_bord_s);   % radius of border points on the sample-immersion medium (cover slip) interface
    
    x_bord_si=ro_bord_si.*cos(fi_bord);      % "x" coordinates of border points on the sample-immersion medium (cover slip) interface
    y_bord_si=ro_bord_si.*sin(fi_bord);      % "y" coordinates of border points on the sample-immersion medium (cover slip) interface
    
    % calculating sampling point coordinates on the sample-immersion
    % interface
    
    
    % interpolating the ray angles to get them for the whole aperture, not
    % just for the intersection
    theta_i(:,sampling_index_y)=interp1(iter_coords, theta_iter_i, r_samp_i(:,sampling_index_y), 'pchip').*circ;
    theta_g(:,sampling_index_y)=interp1(iter_coords, theta_iter_g, r_samp_i(:,sampling_index_y), 'pchip').*circ;
    theta_s(:,sampling_index_y)=interp1(iter_coords, theta_iter_s, r_samp_i(:,sampling_index_y), 'pchip').*circ;
    
    % in order to determine the power transmitted by each ray, the surface
    % elements associated with each ray needs to be calculated
    % these surface elements are calculated from the coordinates of rays
    % belonging to the border points
    % the size of the surface elements needs to be calculated on the
    % sample-immersion medium (cover slip) interface and then the power
    % transmitted by the rays can be determined from the dipole radiation
    
    h_s=zeros(sampling_number_x, sampling_number_y);         % size if surface elements in nm^2 belonging to each ray in the sample medium near the sample-cover slip interface
    % calculating the area of quadrangles determined by each border point
    
    for x_idx=sampling_index_x
        for y_idx=sampling_index_y
            % the coordinates of the four vertices of the actual quadrangle
            a=[x_bord_si(x_idx, y_idx),y_bord_si(x_idx, y_idx),0];
            b=[x_bord_si(x_idx+1, y_idx),y_bord_si(x_idx+1, y_idx),0];
            c=[x_bord_si(x_idx, y_idx+1),y_bord_si(x_idx, y_idx+1),0];
            d=[x_bord_si(x_idx+1, y_idx+1),y_bord_si(x_idx+1, y_idx+1),0];
            
            % the area of the quadrangle which belongs to the actual ray
            % and is perpendicular to it
            h_s(x_idx, y_idx)=0.5*(norm(cross(a-b,a-c))+norm(cross(d-b,d-c)))*...
                cos(theta_s(x_idx, y_idx));
        end
    end
    
    dx_ap=(max(x_bord_tube_norm_int)-min(x_bord_tube_norm_int))*aperture_rad_tube/(border_number_x-1);    % constant number, "x" length belonging to rays on the aperture's plane
    dy_ap=(max(y_bord_tube_norm_int)-min(y_bord_tube_norm_int))*aperture_rad_tube/(border_number_y-1);    % constant number, "y" length belonging to of rays on the aperture's plane
    
    % unit surface element ratios between the objective and tube lens apertures as the apodizing function (or whatever we call it, it would be sqrt(cos(theta_i)) in a non-stratified medium with uniformly spaced rays);
    ray_apodizing_func=sqrt(h_s/(dx_ap*dy_ap));
    
    % transmission coefficient of different surfaces for individual rays
    t_s_sg=2*n_s*cos(theta_s)./(n_s*cos(theta_s)+n_g*cos(theta_g));
    t_p_sg=2*n_s*cos(theta_s)./(n_s*cos(theta_g)+n_g*cos(theta_s));
    t_s_gi=2*n_g*cos(theta_g)./(n_g*cos(theta_g)+n_i*cos(theta_i));
    t_p_gi=2*n_g*cos(theta_g)./(n_g*cos(theta_i)+n_i*cos(theta_g));
    
    % in case of total internal reflection
    t_s_sg(imag(t_s_sg)~=0)=0;
    t_p_sg(imag(t_p_sg)~=0)=0;
    t_s_gi(imag(t_s_gi)~=0)=0;
    t_p_gi(imag(t_p_gi)~=0)=0;
    
    % total transmission coefficients for the individual rays
    t_s=t_s_sg.*t_s_gi;
    t_p=t_p_sg.*t_p_gi;
    
    dist_si=d_s*sqrt(tan(theta_s).^2+1);   % distance of object and the sampled points on the sample-immersion medium (cover slip) interface
    
    % calculating the constant functions for determining the electric field
    % on the aperture
    switch psf_type
        case 'scalar'
            h_mod=1.0545718*1E-34;      % modified Planck's constant in SI units
            c=299792458*10^9;           % speed of light in "m/s"
            eps_0=8.854187817620*1E-12;     % vacuum permeability in SI units
            
            % calculating the electric field amplitude on the sample-immersion interface
            % The apodizing function convert this into the electric
            % field amplitude on the aperture
            E_si=sqrt(em_N_ofphotonpersec_mean*h_mod*k*c*1/(4*pi)*(2/(c*eps_0)))./dist_si;
            
            E_2_i=E_si.*circ.*ray_apodizing_func.*(t_s+t_p)/2;
            
            % changing the intensity distribution on the tube lens to
            % be corresponding with the various PSF models while
            % keeping the calculated optical power
            switch PSF_model
                case {'custom_obj','custom_tube'}
                    if aperture_sampling_fill_bool
                        int_mat=m_gen_PSF_num_int_mat( int_method, sampling_number_x, sampling_number_y );
                        % calculating the amplitude and phase of the electric field at the aperture
                        scalar_aperture_func=E_2_i.*int_mat;       % aperture function without any aberration)
                    else
                        % calculating the amplitude and phase of the electric field at the aperture
                        scalar_aperture_func=E_2_i;       % aperture function without any aberration
                    end
                case {'Airy', 'Gibson and Lanni'}
                    if aperture_sampling_fill_bool
                        int_mat=m_gen_PSF_num_int_mat( int_method, sampling_number_x, sampling_number_y );
                        scalar_aperture_func=sqrt(circ*sum(sum(abs(E_2_i).^2))/sum(sum(circ))).*int_mat;
                    else
                        scalar_aperture_func=sqrt(circ*sum(sum(abs(E_2_i).^2))/sum(sum(circ)));
                    end
                case {'Pankajakshan'}
                    apodizing_func=sqrt(cos(theta_i));
                    if aperture_sampling_fill_bool
                        int_mat=m_gen_PSF_num_int_mat( int_method, sampling_number_x, sampling_number_y );
                        scalar_aperture_func=sqrt(abs(circ.*apodizing_func).^2*sum(sum(abs(E_2_i).^2))/sum(sum(abs(circ.*apodizing_func).^2))).*int_mat;
                    else
                        scalar_aperture_func=sqrt(abs(circ.*apodizing_func).^2*sum(sum(abs(E_2_i).^2))/sum(sum(abs(circ.*apodizing_func).^2)));
                    end
                case {'modified Gibson and Lanni'}
                    mod_apodizing_func=sqrt(cos(theta_i)).*(t_s+t_p)/2;
                    if aperture_sampling_fill_bool
                        int_mat=m_gen_PSF_num_int_mat( int_method, sampling_number_x, sampling_number_y );
                        scalar_aperture_func=sqrt(abs(circ.*mod_apodizing_func).^2*sum(sum(abs(E_2_i).^2))/sum(sum(abs(circ.*mod_apodizing_func).^2))).*int_mat;
                    else
                        scalar_aperture_func=sqrt(abs(circ.*mod_apodizing_func).^2*sum(sum(abs(E_2_i).^2))/sum(sum(abs(circ.*mod_apodizing_func).^2)));
                    end
            end
            
        case 'vectorial'
            
            % unit vector components in Descartes coordinate system parallel with the "s" field component
            e_s_x=cos(fi_samp+pi/2);
            e_s_y=sin(fi_samp+pi/2);
            e_s_z=0;
            
            % unit vector components in Descartes coordinate system parallel with the "p" field component
            e_p_s_x=cos(fi_samp).*sin(theta_s+pi/2);
            e_p_s_y=sin(fi_samp).*sin(theta_s+pi/2);
            e_p_s_z=cos(theta_s+pi/2);
            
            e_p_a_x=cos(fi_samp);
            e_p_a_y=sin(fi_samp);
            e_p_a_z=0;
            % same notation as in "Super-resolution orientation estimation and localization of fluorescent dipoles using 3-D steerable filters"
            
            eps_0=8.854187817620*10^-12;    % vacuum permittivity in Si units
            h_mod=1.0545718*10^-34;         % modified Planck's constant in SI units
            em_N_ofphotonpersec_max=3*em_N_ofphotonpersec_mean;     % number of emitted photons per second when the dipole orientation and the axciting fild vector are parallel
            % calculating the dipole moment based on the total power radiated by a dipole
            dipole_moment=sqrt(em_N_ofphotonpersec_max*h_mod*12*pi*eps_0/k^3);
            % taken from: % https://en.wikipedia.org/wiki/Dipole#Dipole_radiation
            
            % calculating the maximum electric field amplitude radiated by
            % a dipole (electric field without the orientation dependent
            % part) on the sample-cover-slip interface as it would be in
            % vacuum (this does not affects the power transmitted by the
            % rays)
            E_si_0=dipole_moment/(4*pi*eps_0)*k^2./dist_si;
            % taken from: % https://en.wikipedia.org/wiki/Dipole#Dipole_radiation
            % and placed the dipole in the immersion medium intead of vacuum
            
            % constants for determining the "s" and "p" polarized electric field components
            C_s_0=t_s.*ray_apodizing_func.*circ.*E_si_0;
            C_p_0=t_p.*ray_apodizing_func.*circ.*E_si_0;
            
            % changing the intensity distribution on the tube lens to
            % be corresponding with the various PSF models while
            % keeping the calculated optical power
            switch PSF_model
                case {'custom_obj','custom_tube'}
                    % constants for determining the "s" and "p" polarized electric field components
                    C_s=C_s_0;
                    C_p=C_p_0;
                case 'Richards and Wolf'
                    % For anisotropic PSF this kind of correction should be
                    % done after the dipole orientation was taken into
                    % account. It causes photon number devianve between the
                    % 'custom_obj' and 'Richards and Wolf' PSF models
                    % especially for dipoles with parrallel orientation to
                    % the optical axis. It would require significant
                    % rewrite, I won't do it and the photon number deviance
                    % between the 'custom_obj' and 'Richards and Wolf' is
                    % within 10% for all directions.
                    mod_apodizing_func_s=sqrt(cos(theta_i)).*t_s;
                    mod_apodizing_func_p=sqrt(cos(theta_i)).*t_p;
                    C_s=sqrt(abs(circ.*mod_apodizing_func_s).^2*sum(sum(abs(C_s_0).^2))/sum(sum(abs(circ.*mod_apodizing_func_s).^2)));
                    C_p=sqrt(abs(circ.*mod_apodizing_func_p).^2*sum(sum(abs(C_p_0).^2))/sum(sum(abs(circ.*mod_apodizing_func_p).^2)));
            end
            
            if aperture_sampling_fill_bool
                int_mat=m_gen_PSF_num_int_mat( int_method, sampling_number_x, sampling_number_y );
                % contribution of the different dipol moment components to "s"
                % component of the electric field and it's contribution to the
                % "x" and "y" electric field componenets on the image plane
                dipol_comp.s_x_x=e_s_x.*e_s_x.*C_s.*int_mat;
                dipol_comp.s_y_x=e_s_y.*e_s_x.*C_s.*int_mat;
                dipol_comp.s_x_y=e_s_x.*e_s_y.*C_s.*int_mat;
                dipol_comp.s_y_y=e_s_y.*e_s_y.*C_s.*int_mat;
                
                % contribution of the different dipol moment components to "p"
                % component of the electric field and it's contribution to the
                % "x" and "y" electric field componenets on the image plane
                dipol_comp.p_x_x=e_p_s_x.*e_p_a_x.*C_p.*int_mat;
                dipol_comp.p_y_x=e_p_s_y.*e_p_a_x.*C_p.*int_mat;
                dipol_comp.p_z_x=e_p_s_z.*e_p_a_x.*C_p.*int_mat;
                dipol_comp.p_x_y=e_p_s_x.*e_p_a_y.*C_p.*int_mat;
                dipol_comp.p_y_y=e_p_s_y.*e_p_a_y.*C_p.*int_mat;
                dipol_comp.p_z_y=e_p_s_z.*e_p_a_y.*C_p.*int_mat;
            else
                % contribution of the different dipol moment components to "s"
                % component of the electric field and it's contribution to the
                % "x" and "y" electric field componenets on the image plane
                dipol_comp.s_x_x=e_s_x.*e_s_x.*C_s;
                dipol_comp.s_y_x=e_s_y.*e_s_x.*C_s;
                dipol_comp.s_x_y=e_s_x.*e_s_y.*C_s;
                dipol_comp.s_y_y=e_s_y.*e_s_y.*C_s;
                
                % contribution of the different dipol moment components to "p"
                % component of the electric field and it's contribution to the
                % "x" and "y" electric field componenets on the image plane
                dipol_comp.p_x_x=e_p_s_x.*e_p_a_x.*C_p;
                dipol_comp.p_y_x=e_p_s_y.*e_p_a_x.*C_p;
                dipol_comp.p_z_x=e_p_s_z.*e_p_a_x.*C_p;
                dipol_comp.p_x_y=e_p_s_x.*e_p_a_y.*C_p;
                dipol_comp.p_y_y=e_p_s_y.*e_p_a_y.*C_p;
                dipol_comp.p_z_y=e_p_s_z.*e_p_a_y.*C_p;
            end
    end
    
    %  calculating the Lanni-Gibson aberration term caused by the stratified medium
    if aperture_sampling_fill_bool
        NA_LG=n_i*sin(max(max(theta_i)));
    elseif ~aperture_sampling_fill_bool
        NA_LG=n_i*sin(max(theta_iter_i));
    end
    
    switch PSF_model
        case {'Airy'}
            Lanni_Gibson_ab=0;      % no Gibson and Lanni aberration
        case {'custom_obj', 'custom_tube', 'Gibson and Lanni', 'Pankajakshan', 'modified Gibson and Lanni', 'Richards and Wolf'}
            
            % calculate Gibson and Lanni aberration
            Lanni_Gibson_ab_orig=m_gen_PSF_aberration('Lanni_Gibson', params, ro_samp_tube_2, circ, delta_d_i, NA_LG);
            
            % calculating"normalized" aperture function for different PSF
            % models, in case of vectorial models the dipole orientation is
            % not taken into account for simplicity, and it should not
            % cause much error
            switch PSF_model
                case {'custom_obj','custom_tube'}
                    norm_aperture_func=circ.*ray_apodizing_func.*(t_s+t_p)/2./dist_si;
                case {'Gibson and Lanni'}
                    norm_aperture_func=circ;
                case {'Pankajakshan'}
                    apodizing_func=sqrt(cos(theta_i));
                    norm_aperture_func=circ.*apodizing_func;
                case {'modified Gibson and Lanni', 'Richards and Wolf'}
                    apodizing_func=sqrt(cos(theta_i));
                    norm_aperture_func=circ.*apodizing_func.*(t_s+t_p);
                otherwise
                    error('Something is wrong with "PSF_model".')
            end
            
            
            % Lanni and Gibson aberration can shifth the focal point (where
            % the PSF has minimal width). Did not found anything about how
            % "delta_d_i" should be choosen to elimininate this effect.
            % Corrected with minimalizing with either minimalizing the PSF
            % variance ("variance_delta_d_i", "variance_defocus") or with
            % fitting defocus with empirical constants.
            switch z_correction_method
                case 'variance_delta_d_i'
                    % Minimizing the PSF variance with immersion thickness
                    % change
                    
                    Lanni_Gibson_ab=Lanni_Gibson_ab_orig;
                case 'variance_defocus'
                    % Minimizing the PSF variance with defocus
                    
                    sampling_number_var=256;
                    
                    ro_tube_var=(0.5:sampling_number_var-0.5)'/sampling_number_var;        % normalized radii values of the tube lens' aperture
                    ro_det_var=(0.5:sampling_number_var-0.5)'/sampling_number_var;         % normalized radii values of the imaged area
                    
                    % initial guess for the ray angles belonging to the intersection border points
                    theta_edge_i_0=min(atan(abs(aperture_rad_obj)/(d_i+delta_d_i+d_s)), theta_i_max*theta_max_ratio);     % initial guess for the ray angles, required for the iteration method
                    
                    switch ray_angle_iteration_method
                        case 'matlab'
                            options = optimset('Display','off', 'FinDiffType', 'central');        % do not display iteration output
                            N_fsolve_loop=10;       % maximal number of loops for the iteration of "theta_i_ap_edge", inner loops, should be enough
                            fsolve_loop=1;       % variable for the "while" loop
                            while fsolve_loop<=N_fsolve_loop
                                iter_success_bool=true;    % boolean for storing whether all the iterations were successful
                                theta_edge_i_0_max=theta_i_max*theta_max_ratio;             % let this be the maximal initial ray angle, should be smaller than the angle of total internal refraction
                                theta_edge_i_0_iter=theta_edge_i_0;                         % initial ray angle for the iteration
                                theta_edge_i_0_iter(theta_edge_i_0_iter>theta_edge_i_0_max)=theta_edge_i_0_max;   % if any of the initial guess exceeds the maximal value...
                                
                                [theta_edge_i, ~, exitflag]=fsolve(@(theta) ray_angles(theta, abs(aperture_rad_obj)), theta_edge_i_0_iter, options);   % iteration for getting the ray angles
                                % examining whether the iteration was successfull for a given ray
                                if exitflag>0       % if the iteration was successful
                                else                % if the iteration was not successful
                                    iter_success_bool=false;    % do not break out of the "while" loop
                                end
                                
                                % if the iterations were successful for all angles...
                                if iter_success_bool
                                    break;           % break out of the loop
                                end
                                % decrease the "theta_bord_i_0_max" and "theta_i_max" difference in
                                % the next loop
                                fsolve_loop=fsolve_loop+1;
                                theta_max_ratio=1-(1-theta_max_ratio)/2;
                            end
                        case 'custom'
                            [theta_edge_i, ~, exitflag]=ray_angles_iteration(theta_edge_i_0, theta_i_max, 0, abs(aperture_rad_obj), PSF_model);
                            % examining whether the iteration was successfull for a given ray
                            if exitflag>0       % if the iteration was successful
                            else                % if the iteration was not successful
                                iter_success_bool=false;
                            end
                    end
                    
                    if ~iter_success_bool
                        error('no successfor ray angle')
                    end
                    
                    switch apodization_approx_method
                        case 'empirical'
                            apodizing_func_PSF_variance=cos(theta_edge_i*ro_tube_var).^1.5;           % this fits relatively well to the product of the apodizing function and the transmission coefficients (modified apodizing function)
                            %apodizing_func=cos(atan(abs(aperture_rad_obj*ro_tube_iter)/(delta_d_i))).^0.5;
                            %apodizing_func_PSF_variance=1;
                        case 'physical approx'
                            apodizing_func_PSF_variance=apodizing_func_approx_cacl(theta_edge_i, abs(aperture_rad_obj));
                    end
                    
                    NA_LG=n_i*sin(theta_edge_i);    % numerical aperture in Gibson and Lanni aberration
                    
                    LG_0=d_s*n_s+delta_d_i*n_i;         % Lanni and Gibson aberration at the optical axis
                    LG_edge=(d_s*sqrt(n_s^2-NA_LG^2*1)+delta_d_i*sqrt(n_i^2-NA_LG^2*1));       % Lanni and Gibson aberration at the aperture edge
                    
                    defocus_0=(LG_edge-LG_0)/8;     % let this be the initial value
                    
                    [defocus,~,exitflag_fit]=fminsearch(@(defocus_iter) PSF_defocus_variance(defocus_iter), defocus_0);
                    
                    if exitflag_fit>0       % if the iteration was successful
                    else                % if the iteration was not successful
                        error('Could not minimalize the PSF variance. Please decrease the numerical aperture or the simple medium thickness.')
                    end
                    
                    Lanni_Gibson_ab=Lanni_Gibson_ab_orig-defocus*ro_samp_tube_2.*circ;
                case 'empirical'
                    % "delta_d_i" was approximated with paraxial rays and
                    % the Lanni and Gibson aberration was corrected with a
                    % spherical wavefront determined with surface fitting and
                    % empirical constants. It is not the most elegant and correct
                    % solution, but works with bot low and high numerical apertures
                    % and should not cause much error.
                    
                    error_power=0.45;      % power for fitting error calculation, empirical value
                    
                    % integration matrix, for fitting error calculation
                    if aperture_sampling_fill_bool
                        int_mat=m_gen_PSF_num_int_mat( int_method, sampling_number_x, sampling_number_y );
                    else
                        int_mat=ones( sampling_number_x, sampling_number_y );
                    end
                    
                    weight_power=0.35;          % power for weighting factor calculation, empirical value
                    
                    % calculating weighting factor for fitting error calculation
                    error_weights=(abs(norm_aperture_func)*sum(sum(circ))/sum(sum(abs(norm_aperture_func)))).^weight_power;
                    
                    LG_0=d_s*n_s+delta_d_i*n_i;         % Lanni and Gibson aberration at the optical axis
                    LG_edge=(d_s*sqrt(n_s^2-NA_LG^2*1)+delta_d_i*sqrt(n_i^2-NA_LG^2*1));       % Lanni and Gibson aberration at the aperture edge
                    
                    tolerance_const=1/sum(sum(abs((Lanni_Gibson_ab_orig-LG_0).*circ).^error_power))/1E-6;   % constant for the surface fitting, "scaling" the iteration tolerance
                    
                    % let these be the intitial guess values for the "a+b*x^2" equation
                    fit_var_0=[LG_0,(LG_edge-LG_0)];
                    
                    options = optimset('Display','off');        % do not display iteration output
                    
                    % fitting the surface with "fminsearch", "fmincon" could be another solution
                    [fit_calc_var,~,exitflag_fit]=fminsearch(@(fit_var) defocus_fitting(fit_var), fit_var_0, options);
                    
                    if exitflag_fit>0       % if the iteration was successful
                    else                % if the iteration was not successful
                        error('Could not correct the defocus of gibson and Lanni aberration. Please decrease the numerical aperture.')
                    end
                    
                    % the corrected Lanni and Gibson aberration
                    Lanni_Gibson_ab=Lanni_Gibson_ab_orig-(fit_calc_var(1)+fit_calc_var(2)*ro_samp_tube_2).*circ;
            end 
    end
    
    % calculating non-position dependent aberrations (except Lanni and Gibson)
    constant_ab=m_gen_PSF_aberration('Zernike_const', params, x_samp_tube_norm, y_samp_tube_norm, ro_samp_tube_2)...
        +m_gen_PSF_aberration('astigmatism', params, y_samp_tube_norm)...
        +m_gen_PSF_aberration('Fourier_mask', params)...
        +Lanni_Gibson_ab;
    
    aperture_func_bool=true;
    
    % testing the different variables
    %figure()
    %plot(theta_bord_int_i_0)
    %figure()
    %plot(real(theta_bord_i_iter))
    %figure()
    %plot(imag(theta_bord_i_iter))
    %figure()
    %mesh(real(h_s.*circ))
    %figure()
    %plot(theta_s(64, :))
    %figure()
    %mesh(t_s.^2.*circ)
    %figure()
    %mesh(t_p.^2.*circ)
    %figure()
    %mesh(apodizing_func.^2.*circ)
    %figure()
    %mesh(E_si_0.^2.*circ)
    %figure()
    %mesh(abs(C_s).^2)
    %figure()
    %mesh(abs(C_p).^2)
    
end

%calculating the total phase front aberration

[pos_dependent_ab]=m_gen_PSF_aberration('Zernike_pos' ,params, x_samp_tube_norm, y_samp_tube_norm, ro_samp_tube_2);     % getting the Zernike part of the aberration function

aberration=(pos_dependent_ab+constant_ab);      % the total phase front aberration on the aperture


%figure()
%mesh(aberration)
%figure()
%mesh(pos_dependent_ab)
%figure()
%mesh(constant_ab)
%figure()
%mesh(pos_dependent_ab+constant_ab)

% getting the aperture function for the scalar or vectorial diffraction
% calculations
switch psf_type
    case 'scalar'       % for scalar diffraction calculation
        aperture_func{1}=scalar_aperture_func.*exp(1i*k*aberration);   % the complex aperture function
        
        
        % for testing the electric field on the apertures
        %figure()
        %mesh(scalar_aperture_func)
        %figure()
        %mesh(abs(aperture_func{1}))
        %figure()
        %mesh(real(aperture_func{1}))
        %figure()
        %mesh(imag(aperture_func{1}))
        
    case 'vectorial'    % for vectorial diffraction calculation
        switch anisotropy_bool
            case false      % fixed dipole orientation
                % orientation of the dipole
                fi_dipol=orientation(1);
                theta_dipol=orientation(2);
                
                % components of the dipole moment
                p_x=cos(fi_dipol)*sin(theta_dipol);
                p_y=sin(fi_dipol)*sin(theta_dipol);
                p_z=cos(theta_dipol);
                
                % normalized electric field vector
                E=params.numcalc.E;
                
                % |E*p| term, photon number should be corrected with it, em_N_ofphotonpersec=|E*p|^2*em_N_ofphotonpersec_max
                Ep = abs(E(1)*p_x...
                    +E(2)*p_y...
                    +E(3)*p_z);
                
                % "x" component of the complex electric field on the aperture
                aperture_func{1}=(...
                    p_x*(dipol_comp.s_x_x+dipol_comp.p_x_x)+...
                    p_y*(dipol_comp.s_y_x+dipol_comp.p_y_x)+...
                    p_z*dipol_comp.p_z_x...
                    )*Ep.*exp(1i*k*aberration);
                
                % "y" component of the complex electric field on the aperture
                aperture_func{2}=(...
                    p_x*(dipol_comp.s_x_y+dipol_comp.p_x_y)+...
                    p_y*(dipol_comp.s_y_y+dipol_comp.p_y_y)+...
                    p_z*dipol_comp.p_z_y...
                    )*Ep.*exp(1i*k*aberration);
                
            case true       % mobile (within a cone) dipole orientation
                
                aperture_func{1}=(dipol_comp.s_x_x+dipol_comp.p_x_x).*exp(1i*k*aberration);
                aperture_func{2}=(dipol_comp.s_y_x+dipol_comp.p_y_x).*exp(1i*k*aberration);
                aperture_func{3}=(dipol_comp.p_z_x).*exp(1i*k*aberration);
                aperture_func{4}=(dipol_comp.s_x_y+dipol_comp.p_x_y).*exp(1i*k*aberration);
                aperture_func{5}=(dipol_comp.s_y_y+dipol_comp.p_y_y).*exp(1i*k*aberration);
                aperture_func{6}=(dipol_comp.p_z_y).*exp(1i*k*aberration);
        end
        
end

end

