function [ S ] = m_gen_PSF_cone_integration( params )
% This function performs numerical integration for different dipole
% oriantations on a hard-edged cone in order to describe anisotrophy. The
% numerical integration is evaluated in both case when rotation diffusion
% time constant is much smaller and much greater than the flourescence
% lifetime.
% Mathematical descriptions and formulas:
% Supporting information of "Rotational Mobility of Single Molecules Affects Localization Accuracy in Super-Resolution Flourescence Microscopy"
% "Enhanced DNA imaging using super-resolution microscopy and simultaneous single-molecule orientation measurements: supplementary material"

alpha=params.psf.an_cone_angle*pi/180;             % cone half angle 
theta_0=params.mol.orientation(2);     % polar angle of dipole moment
fi_0=params.mol.orientation(1);        % azimuth angle of dipole moment
E=params.numcalc.E;                         % normalized electric field direction vector

if alpha<=0 || alpha>pi
   error('Please give a valid cone alpha value.')
end

% polar and azimuth angles in the original coordinate system expressed with
% the polar and azimuth angles in the rotated coordinate system
theta_orig = @(theta,fi) acos(cos(theta).*cos(theta_0)-sin(theta).*sin(theta_0).*cos(fi));
fi_orig = @(theta,fi) atan2(sin(theta).*(cos(fi).*sin(fi_0).*cos(theta_0)+sin(fi).*cos(fi_0))+cos(theta).*sin(fi_0).*sin(theta_0),...
    sin(theta).*(cos(fi).*cos(fi_0).*cos(theta_0)-sin(fi).*sin(fi_0))+cos(theta).*cos(fi_0).*sin(theta_0));

% functions describing the polar and azimuth angle dependent part of
% (a*p_x+b*p_y+c*p_z)^2 ("a", "b" and "c" polar and azimuth angle independent)
%F_p_x_x = @(theta,fi) sin(theta_orig(theta,fi)).^2.*cos(fi_orig(theta,fi)).^2;
%F_p_y_y = @(theta,fi) sin(theta_orig(theta,fi)).^2.*sin(fi_orig(theta,fi)).^2;
%F_p_z_z = @(theta,fi) cos(theta_orig(theta,fi)).^2;
%F_p_x_y = @(theta,fi) sin(theta_orig(theta,fi)).^2.*cos(fi_orig(theta,fi)).*sin(fi_orig(theta,fi));
%F_p_x_z = @(theta,fi) sin(theta_orig(theta,fi)).*cos(fi_orig(theta,fi)).*cos(theta_orig(theta,fi));
%F_p_y_z = @(theta,fi) sin(theta_orig(theta,fi)).*sin(fi_orig(theta,fi)).*cos(theta_orig(theta,fi));

% gains about 3% improvement in speed by rewriting to this from using
% trigonometric identities
F_p_x_x = @(theta,fi) (1-cos(2*theta_orig(theta,fi)))/2.*(1+cos(2*fi_orig(theta,fi)))/2;
F_p_y_y = @(theta,fi) (1-cos(2*theta_orig(theta,fi)))/2.*(1-cos(2*fi_orig(theta,fi)))/2;
F_p_z_z = @(theta,fi) (1+cos(2*theta_orig(theta,fi)))/2;
F_p_x_y = @(theta,fi) (1-cos(2*theta_orig(theta,fi)))/2.*sin(2*fi_orig(theta,fi))/2;
F_p_x_z = @(theta,fi) sin(2*theta_orig(theta,fi))/2.*cos(fi_orig(theta,fi));
F_p_y_z = @(theta,fi) sin(2*theta_orig(theta,fi))/2.*sin(fi_orig(theta,fi));

% function describing the |E*p|^2 term
F_Ep2 = @(theta,fi) abs(E(1).*sin(theta_orig(theta,fi)).*cos(fi_orig(theta,fi))...
    +E(2).*sin(theta_orig(theta,fi)).*sin(fi_orig(theta,fi))...
    +E(3).*cos(theta_orig(theta,fi))).^2;
% the volume element for integrating in spherical coordinate system
F_volume_element = @(theta) sin(theta);

% functions to integrate

% functions to integrate for fast rotation
F_p_x_x_int = @(theta,fi) F_p_x_x(theta,fi).*F_volume_element(theta);
F_p_y_y_int = @(theta,fi) F_p_y_y(theta,fi).*F_volume_element(theta);
F_p_z_z_int = @(theta,fi) F_p_z_z(theta,fi).*F_volume_element(theta);
F_p_x_y_int = @(theta,fi) F_p_x_y(theta,fi).*F_volume_element(theta);
F_p_x_z_int = @(theta,fi) F_p_x_z(theta,fi).*F_volume_element(theta);
F_p_y_z_int = @(theta,fi) F_p_y_z(theta,fi).*F_volume_element(theta);

F_Ep2_int = @(theta,fi) F_Ep2(theta,fi).*F_volume_element(theta);

% functions to integrate for slow rotation
F_Ep_x_x_int = @(theta,fi) F_p_x_x(theta,fi).*F_Ep2(theta,fi).*F_volume_element(theta);
F_Ep_y_y_int = @(theta,fi) F_p_y_y(theta,fi).*F_Ep2(theta,fi).*F_volume_element(theta);
F_Ep_z_z_int = @(theta,fi) F_p_z_z(theta,fi).*F_Ep2(theta,fi).*F_volume_element(theta);
F_Ep_x_y_int = @(theta,fi) F_p_x_y(theta,fi).*F_Ep2(theta,fi).*F_volume_element(theta);
F_Ep_x_z_int = @(theta,fi) F_p_x_z(theta,fi).*F_Ep2(theta,fi).*F_volume_element(theta);
F_Ep_y_z_int = @(theta,fi) F_p_y_z(theta,fi).*F_Ep2(theta,fi).*F_volume_element(theta);


% accuracy of the numerical integration
options.AbsTol=1e-4;
options.RelTol=1e-4;

norm_cone=1/(2*pi*(1-cos(alpha)));  % normaliziton factor for the integrations, reciprocal of the area under the integrated cone on an unit sphere

% performing the numerical integrations

% for fast rotations
S.fast.x_x = integral2(F_p_x_x_int,0,alpha,0,2*pi, options)*norm_cone;
S.fast.y_y = integral2(F_p_y_y_int,0,alpha,0,2*pi, options)*norm_cone;
S.fast.z_z = integral2(F_p_z_z_int,0,alpha,0,2*pi, options)*norm_cone;
S.fast.x_y = integral2(F_p_x_y_int,0,alpha,0,2*pi, options)*norm_cone;
S.fast.x_z = integral2(F_p_x_z_int,0,alpha,0,2*pi, options)*norm_cone;
S.fast.y_z = integral2(F_p_y_z_int,0,alpha,0,2*pi, options)*norm_cone;

S.fast.Ep2 = integral2(F_Ep2_int,0,alpha,0,2*pi, options)*norm_cone;

% for slow rotations
S.slow.x_x = integral2(F_Ep_x_x_int,0,alpha,0,2*pi, options)*norm_cone;
S.slow.y_y = integral2(F_Ep_y_y_int,0,alpha,0,2*pi, options)*norm_cone;
S.slow.z_z = integral2(F_Ep_z_z_int,0,alpha,0,2*pi, options)*norm_cone;
S.slow.x_y = integral2(F_Ep_x_y_int,0,alpha,0,2*pi, options)*norm_cone;
S.slow.x_z = integral2(F_Ep_x_z_int,0,alpha,0,2*pi, options)*norm_cone;
S.slow.y_z = integral2(F_Ep_y_z_int,0,alpha,0,2*pi, options)*norm_cone;
end
