function [labels, N_mol_cont, link_vect_sign] = m_gen_linker( varargin )
% This function allocates memory for arrays containing contact point
% positions, linker vectors and linker orientations. It also calls the
% function which calcutes the initial linker vectors. Eventaully calculates
% the values for "link_vect_sign" vector. With this vector the distribution
% can be mapped from half sphere to full sphere. Patterns which do not have
% surfaces (array, octagon patterns) will have uniform linker vector
% distribution on a full sphere. If no surface normals were given for mesh
% pattern (e.g. star pattern) the linker vector distribution shall have
% mirror symmetry on the "x-y" axis.

% PARENTS:  m_gen_mol_coord_axons.m
%           m_gen_mol_coord_vesicles.m
%           m_gen_mol_coord_points.m
%           m_gen_mol_coord_tubules.m
%           m_gen_mol_coord_mesh.m
% CHILD: m_gen_linker_vect.m

% VARIABLES
% N_cont: total number of contact points
% params: structure containing sample, dye and acquisition parameters
% N_mol_cont: number of dye molecules belonging to individual contact points
% N_mol: total number of dye molecules
% labels: structure containing contact point and dye informations
% link_vect_sign: vector containing information about the actual linker vector should be mirrored or not
% mesh_coords: structure containing mesh coordinates, if it does not have "norm_vect" field, linkers should be mirrored

N_cont=varargin{1};     % total number of contact points
params=varargin{2};     % sample, dye and acquisition parameters

% parameters for the binomial distribution
N_mean=params.dye.N_of_labels_per_epitope;      % mean number of dye molecules on epitopes (contact point)
N_var=params.dye.var_N_of_labels_per_epitope;   % variance of the  number of dye molecules on epitopes

P=1-N_var/N_mean;       % success probability of each trial
if P<=0
    P=0;        % count it as zero
    N_trials=0;
    disp('Too large sigma was given for the epitopes...')
% if "N_mean" is zero
elseif isnan(P)
    P=0;
    N_trials=0;
% if P has meaningful value
else
    N_trials=round(N_mean/P);    % number of trials in the binomial distribution
end

N_mol_cont=m_gen_random('binomial', N_trials, P, N_cont, 1);       % number of dye molecules on individual contact points, follows binomial distribution
%N_mol_cont=binornd(N_trials,P,N_cont,1);       % number of dye molecules on individual contact points, follows binomial distribution
N_mol=sum(N_mol_cont);          % total number of dye molecules

labels=[];      % output variable
% memory allocation for the contact point coordinates
labels.init_cont_coords=zeros(3, N_cont);
% memory allocation for vectors containing the orientation of the linkers
labels.init_dye_coords=m_gen_linker_vect(N_mol, params);
% memory allocation for the dye oriantations
labels.init_orientation=zeros(2,length(labels.init_dye_coords(1, :)));

% calculating "-1" and "1" values for "link_vect_sign"
switch length(varargin)
    case 2
        switch params.sample.name
            case { 'lattice', 'octagons', 'disks'}       % patterns which does not have surfaces
                link_vect_sign=sign(rand(N_mol,1)-0.5);     % expand distribution to full sphere
                link_vect_sign(link_vect_sign==0)=1;
            case {'lines', 'vesicles', 'rods', 'axons', 'segments'} % patterns which have surfaces
                link_vect_sign=ones(N_mol,1);       % keep surface distribution on half sphere
        end
    case 3
        mesh_coords=varargin{3};
        % if surface vectors were given...
        if isfield(mesh_coords,'norm_vect')
            link_vect_sign=ones(N_mol,1);       % keep surface distribution on half sphere
        % if no surface vectors were given...
        else
            link_vect_sign=sign(rand(N_mol,1)-0.5);     % expand distribution to full sphere
            link_vect_sign(link_vect_sign==0)=1;
        end
end


end

