function [ link_vect ] = m_gen_linker_vect( N_mol, params )
% This function calculates the position of the linkers relative to the
% surface. First creates uniform or normal distribution on a sphere. In 
% case of nrmal distribution it rotates the vectors according to the mean
% binding angle. The mean binding angle defines the rotation of polar
% angles, azimuth rotation angles have uniform distribution. The linker can
% not go "below" the surface.

% PARENT: m_gen_linker.m
% CHILDREN:-

% VAIABLES
% N_mol: total number of dye molecules
% params: structure containing sample, dye and acquisition parameters
% link_rad: radius of the linkers
% theta2: polar angle of the linker
% fi2: azimuth angle of the linker
% link_vect: vector of the linker
% binding_angle: mean value of binding angle
% binding_angle_dev: deviation of binding angle
% theta: mean value of binding angle
% R_y: rotation matrix for the linker around the "y" axis, uniformly distributed rotation
% idx_mol: index of the actual dye molecule
% fi: random azimuth angle of the linker's rotation
% R_z: rotation matrix for the linker around the "z" axis according to mean binding angle
% rand_for_theta2: auxiliary variable for "theta2"
% int_div: variable needed for "folding" the random distribution into the sphere surface
% theta2: polar angle of the unrotated linker following normal distribution
% fi2: azimuth angle of the unrotated linker following uniform distribution
% link_vect_act: vector of the linker, only accepted if it is above the surface

switch params.sample.name
    % use uniform distribution for array and octagon pattern
    case {'lattice', 'octagons'}
        link_rad=params.dye.link_rad;        % radius of the linker
        theta2=acos(1-2*rand(1,N_mol));
        fi2=2*pi*rand(1,N_mol);
        link_vect=[...
            sin(theta2).*cos(fi2);...
            sin(theta2).*sin(fi2);...
            cos(theta2)]*link_rad;
    % use normal distribution for every other pattern
    otherwise
        binding_angle=params.dye.binding_angle/180*pi;    % mean binding angle of the linker
        binding_angle_dev=params.dye.binding_angle_dev/180*pi;    % standard deviation of the binding angle
        link_rad=params.dye.link_rad;        % radius of the linker

        theta=binding_angle; % mean angle between the linkers and the surface normal
        % rotation of the polar angle
        R_y=[cos(theta),0,sin(theta);...
            0,1,0;...
            -sin(theta),0,cos(theta)];

        % memory allocation for for the linker positions
        link_vect=zeros(3,N_mol);

        idx_mol=1;  % index of the actual molecule
        while idx_mol<=N_mol  % goes through the dye molecules
            % calculating the polar angle in normal distribution on the sphere,
            % similarly to the "Inverse transform sampling"
            % rotation of the azimuth angle
            fi=2*pi*rand(1);        % rotation of the linker around the normal vector
            R_z=[cos(fi),-sin(fi),0;...
                sin(fi),cos(fi),0;...
                0,0,1];
            rand_for_theta2=binding_angle_dev*randn(1);
            int_div=floor((rand_for_theta2)/2);
            % this is needed for "folding" the random distribution into the
            % sphere surface
            if mod(int_div,uint8(2))==0
                rand_for_theta2=rand_for_theta2-(2)*(int_div);
                theta2=acos(1-rand_for_theta2);
            elseif mod(int_div,uint8(2))==1
                rand_for_theta2=-rand_for_theta2+(2)*(int_div+1);
                theta2=acos(1-rand_for_theta2);
            else
                disp('Error occured during the binding angle calcution...')
            end
            
            % vector according to the normal distribution with mean binding angle on the sphere
            switch params.sample.name
                case {'axons', 'segments', 'lines'}
                    % for these structures the mean binding angle is
                    % located in the plane defined by the normal and the
                    % line axis vector or the ring tangent vector
                    % do not assume rotational symmetry around the normal
                    % vector
                    link_vect_act=R_y*R_z*[...
                        sin(theta2);...
                        0;...
                        cos(theta2)]*link_rad;
                otherwise
                    %for these structures the azimuth angle has uniform distribution
                    % do assumes rotational symmetry around the normal
                    % vector
                    fi2=2*pi*rand(1);
                    link_vect_act=R_z*R_y*[...
                        sin(theta2)*cos(fi2);...
                        sin(theta2)*sin(fi2);...
                        cos(theta2)]*link_rad;
            end
            % if the linker is not "below" the surface...
            if link_vect_act(3,1)*link_rad*cos(theta)>=0
                link_vect(:,idx_mol)=link_vect_act;
                idx_mol=idx_mol+1; % and leaps to the next dye molecule
            end
        end
end

end

