function output=m_gen_trajs_to_seq(params, trajs)
% This function inspects which dye molecules were active during the
% dirrerent frames. Goes trough the dye molecules in order and finds their
% active states and determines which frames overlap with the actual active
% state. Then it goes trough these frames, and calculates how much of the
% actual frames's photon collecting time overlaps with the active state
% ("delta_t"). Then it calculates the PSF (two PSF in case of polarization
% sensitive measurement) of the dye blinking and adds it to the image using
% its. If the actual active state exceeds the meaurement time (or the frame
% index exceeds the number of captured frames), skips to the next frame
% until reaches the last frame belonging to the active state, then jumps to
% the next active state (there should not be more active state for that dye
% molecule), thus jumps to the next dye molecule (would it not be simpler
% using "brake"?). Finally it adds the autofluorescence backgrond to the
% image.
% Return the image consinting of the blinking's PSFs and autoflurescence
% background and the positions the molecules and the frame indeces during
% the active states.

% VARIABLES
% T_frame: full frame time
% idx_mol: index of the actual dye molecule
% states: the different states belonging to the actual dye molecule
% trajs(idx_mol).t: starting times of the molecule's active states
% t: starting times of the actual molecule's active states
% pos: position of the dye molecule during the actual frame
% idx_active: index of the actual active state
% idx_frame: index of the actual frame
% begin_frame_idx: index of the frame where the actual active state starts
% last_frame_idx: index of the frame where the actual active state ends
% frame_min_time: starting time of the actual frame's photon collection (exposure time)
% frame_max_time: ending time of the actual frame's photon collection (exposure time)
% delta_t: lenght of the active state during the actual frame exposure time
% mol_param.pos: position of the dye molecule during the active state

% global boolean variables used in the PSF generation
global Gaussian_beam_params_bool;
global DFT_matrix_bool;
global aperture_func_bool;
global numerical_calc_cont_bool;

% booleans for deciding whether the persistent variables should be calculated (or recalculated)
Gaussian_beam_params_bool=false;
DFT_matrix_bool=false;
aperture_func_bool=false;
numerical_calc_cont_bool=false;     % it it not required, just for safety reasons

% these variables are used to calculate the PSF for the same blibking event
% only once even if it time duration extends to several frames
persistent PSF_image idx_pix_x idx_pix_y

qeff=params.aq.qeff;
preamp=params.aq.preamp;
frame_rate=params.aq.frame_rate;
frame_N=params.aq.frame_N;          % total number of frames
frame_size=params.aq.frame_size;    % size of frame (number of pixels in a row or a column)
actemgain=params.aq.actemgain;
frame_fill_factor=params.aq.frame_fill_factor;  % ratio of the camera's exposure time and the full frame time
elpercount=params.aq.elpercount;
autofluor_bg=params.aq.autofluor_bg;
polarization_bool=params.meas.polarization_bool;


% memory allocation and initialization
output.output_names='idx_frame	idx_mol	x	y	z	N_photon';
output.output_params=cell(2,1); % array containing frame indices and active molecule positions
N_photon=cell(2,1);
PSF_image_signal=cell(2,1);
if ~polarization_bool   % if non-polarization sensitive measurement is used
    output.frame=cell(1,1);
    output.frame{1}=zeros(frame_size, frame_size, frame_N, 'single');    % occupy memory for one image stack
elseif polarization_bool    % if polarization sensitive measurement is used
    output.frame=cell(2,1);
    output.frame{1}=zeros(frame_size, frame_size, frame_N, 'single');
    output.frame{2}=zeros(frame_size, frame_size, frame_N, 'single');    % occupy memory for two image stacks
end
sum_image=zeros(frame_size);     % sum image for determining the autopluorescence background

drift_coords=m_gen_trajs_drift(params); % coordinate changes caused by the drift for every frame

% calculating the constants with which the PSF should be multiplied (in case of unitary (1s) blinking duration)
countperphoton=qeff*preamp*actemgain/elpercount; % converts the number of hitting the pixel to the signal of the detector
%switch psf_type
%    case 'gaussian'     % in case of Gaussian PSF model
%        countpersec=qeff*preamp*actemgain/elpercount;      % converts the number of hitting the pixel to the signal of the detector
%    case {'scalar', 'vectorial'}        % in case of numerically calculated PSF
%        countpersec=qeff*preamp*actemgain/elpercount; % converts the number of hitting the pixel to the signal of the detector
%end

T_frame=1/frame_rate;    % full frame time

params.mol.ideal_PSF_bool=true;
%[PSF_ideal,~,~, photon_ratio]=m_gen_PSF_calc(params); % the PSF of such a molecule
[PSF_ideal,~,~]=m_gen_PSF_calc(params, false); % the PSF of such a molecule

idx_act_PSF=0;  % index of the PSF (determined by the blinking event and which frame it belongs to)
idx_prev_PSF=0; %  index of the previous PSF

N_mol=length(trajs);
N_mol_rec=1/N_mol;

% calculate the number of blinkng events
N_blinking=0;
for idx_mol=1:N_mol
    % goes through the dye molecules and finds their active states
    states=trajs(idx_mol).state;
    t=trajs(idx_mol).t;
    states(end)=0;
    for idx_active=find(states>0)
        % goes through the active states and determines which frame they belong to
        idx_act_PSF=idx_act_PSF+1;
        begin_frame_idx=floor(t(idx_active)/T_frame)+1;
        last_frame_idx=floor(t(idx_active+1)/T_frame)+1;
        for idx_frame=begin_frame_idx:last_frame_idx
            frame_max_time=T_frame*(idx_frame-1+frame_fill_factor);     % ending time of the frame's photon collection
            frame_min_time=T_frame*(idx_frame-1);                       % starting time of the frame's photon collection
            if t(idx_active)>frame_max_time || idx_frame>frame_N        % if the blinking event starts after the actual (for safety reasons?) or the last frame, skip to next
                continue
            end
            delta_t=min(frame_max_time,t(idx_active+1))-...         % the blinbking event's duration on the frame
                max(frame_min_time,t(idx_active));
            
            if delta_t*1000<T_frame     % if the blinking event is short compared to the frame time, skip to next
                continue
            end
            N_blinking=N_blinking+1;
        end
    end
end
% initializing the matrix containing the blinking  parameters
output.output_params{1}=zeros(N_blinking,6);
output.output_params{1}=zeros(N_blinking,6);


disp('0%')
idx_blinking=0;
for idx_mol=1:N_mol
    % goes through the dye molecules and finds their active states
    states=trajs(idx_mol).state;
    t=trajs(idx_mol).t;
    pos=trajs(idx_mol).pos;
    orientation=trajs(idx_mol).orient;
    states(end)=0;
    for idx_active=find(states>0)
        % goes through the active states and determines which frame they belong to
        idx_act_PSF=idx_act_PSF+1;
        begin_frame_idx=floor(t(idx_active)/T_frame)+1;
        last_frame_idx=floor(t(idx_active+1)/T_frame)+1;
        for idx_frame=begin_frame_idx:last_frame_idx
            % goes through the frames of the active state and adds the PSF to the image
            frame_max_time=T_frame*(idx_frame-1+frame_fill_factor);     % ending time of the frame's photon collection
            frame_min_time=T_frame*(idx_frame-1);                       % starting time of the frame's photon collection
            if t(idx_active)>frame_max_time || idx_frame>frame_N        % if the blinking event starts after the actual (for safety reasons?) or the last frame, skip to next
                continue
            end
            delta_t=min(frame_max_time,t(idx_active+1))-...         % the blinbking event's duration on the frame
                max(frame_min_time,t(idx_active));
            %count=delta_t*countperphoton*photon_ratio;             % constant to multiply the calculated PSF with
            count=delta_t*countperphoton;                           % constant to multiply the calculated PSF with
            params.mol.pos=pos(:,idx_active)+drift_coords(:,idx_frame);  % molecule's position �t the beginning of the frame (assumed the drift if is slowed compared to the frame time)
            params.mol.orientation=orientation(:,idx_active);
            params.mol.ideal_PSF_bool=false;
            
            if delta_t*1000<T_frame     % if the blinking event is short compared to the frame time, skip to next
                continue
            end
          
            idx_blinking=idx_blinking+1;
            if idx_prev_PSF~=idx_act_PSF    % if the PSF has not been calculated for the blinking event
                [PSF_image, idx_pix_x, idx_pix_y]=m_gen_PSF_calc(params, false);     % get the PSF and the affected pixel indices by it
            else
                % do nothing
                
                %for testing purposes, taking into account the effects of drift for the same blinking event
                %[PSF_image, idx_pix_x, idx_pix_y]=m_gen_PSF_calc(params);     % get the PSF the affected pixel indices by it
            end
            
            for idx_pol=1:length(PSF_image)
                N_photon{idx_pol}=sum(sum(PSF_image{idx_pol}))*delta_t;
                PSF_image_signal{idx_pol}=PSF_image{idx_pol}*count;    % multiplying the PSF with a constant to get the signal values
                output.frame{idx_pol}(idx_pix_x, idx_pix_y, idx_frame)=...
                    output.frame{idx_pol}(idx_pix_x, idx_pix_y, idx_frame)+PSF_image_signal{idx_pol};   % the PSF is added to the image
                
                output.output_params{idx_pol}(idx_blinking,:)=[idx_frame, idx_mol, params.mol.pos', N_photon{idx_pol}];
            end
            
            idx_prev_PSF=idx_act_PSF;
        end
    end
    percentage=idx_mol*N_mol_rec;   % percentage of created PSFs
    
    if ( (floor(percentage*10) > floor((percentage-N_mol_rec)*10)))
        X = sprintf('%d%%',floor(percentage*10)*10);    % display the percentage at every '10%'
        disp(X)
    end
end

% calculating the autofluorescence background
if autofluor_bg~=0
    
    % maximal value of the pixelized PSF
    PSF_max=max(max(PSF_ideal{1}))*T_frame*frame_fill_factor*countperphoton;
    
    % calculating the the sum image of the blinkings
    for idx_pol=1:length(output.frame)
        sum_image=sum_image+sum(output.frame{idx_pol},3)/frame_N;       % avaraging for the frames and adding the two (or one) polarization channels
    end
    
    % maximal pixel value on the sum image
    sum_image_max=max(max(sum_image));
    
    % adding the autofluorescence background to the image
    for idx_pol=1:length(output.frame)
        for idx_frame=1:frame_N
            % linear change
            output.frame{idx_pol}(:,:,idx_frame)=output.frame{idx_pol}(:,:,idx_frame)...
                +sum_image*(autofluor_bg(1)+(idx_frame-1)/(frame_N-1)*(autofluor_bg(2)-autofluor_bg(1)))*PSF_max/sum_image_max;
            % exponential change
            %if autofluor_bg(1)>0 && autofluor_bg(2)>0
            %    output.frame{idx_pol}(:,:,idx_frame)=output.frame{idx_pol}(:,:,idx_frame)...
            %        +sum_image*(autofluor_bg(1)*exp(-log(autofluor_bg(1)/autofluor_bg(2))*(idx_frame-1)/(frame_N-1)))*PSF_max/sum_image_max;
            %else
            %    output.frame=output.frame;
            %end
        end
    end
end

% clearing the persistent variables used for PSF generation
clear m_gen_PSF_gauss % it must be cleared as frame size can change in the next simulation
% no need to clear them, just for safety reason:
clear m_gen_PSF_numerical_calc
clear m_gen_PSF_aperture_func
clear m_gen_PSF_DFT
clear m_gen_PSF_pixelise

end